package jp.ac.kagawa_u.eng.wappen.simplejava.nature;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.jdt.internal.compiler.batch.Main;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IOConsole;
import org.eclipse.ui.console.IOConsoleOutputStream;

public class ProjectBuilder extends IncrementalProjectBuilder {

	public static final String BUILDER_ID = "jp.ac.kagawa_u.eng.wappen.simplejava.builder";

	@Override
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor)
	throws CoreException {
		getProject().accept(new IResourceVisitor() {
			@SuppressWarnings({ "restriction", "unchecked" })
			public boolean visit(IResource resource) throws CoreException {
//				System.out.printf("resource name %s%n", resource.getName());
				if (resource instanceof IFile && resource.getName().endsWith(".java")) {
//					System.out.printf("java source path %s%n", resource.getLocation().toString());

					IOConsole console = (IOConsole)resource.getSessionProperty(new QualifiedName(null, "__console"));
					if (console==null) {
						console = new IOConsole("Compiling "+resource.getName(), null);
						resource.setSessionProperty(new QualifiedName(null, "__console"), console);
					} else {
						console.clearConsole();
					}
					ConsolePlugin.getDefault().getConsoleManager().addConsoles(new IConsole[] {console});
					PrintWriter out = new PrintWriter(console.newOutputStream());
					IOConsoleOutputStream err0 = console.newOutputStream();
//					Color errColor = new Color(null, 255, 0, 0);
					// TODO G[oĂ@org.eclipse.swt.SWTException: Invalid thread access
//					err0.setColor(errColor);
					PrintWriter err = new PrintWriter(err0);

					compileJavaProgram(resource, out, err);

					out.println();
					err.println();
					if (console!=null) {
						out.close();
						err.close();
//						errColor.dispose();
					}
				}
				return true;
			}

			@SuppressWarnings({ "unchecked", "restriction" })
			private void compileJavaProgram(IResource resource, PrintWriter out, PrintWriter err) throws CoreException {
				IProject project = resource.getProject();
				ArrayList<IFile> jars = (ArrayList<IFile>)project.getSessionProperty(new QualifiedName(null, "__jars"));
				StringBuilder sb = new StringBuilder();

				if (jars!=null) {
					sb.append(" -classpath \"");
					for(IFile jar : jars) {  
						sb.append(jar.getLocation().toOSString());
						sb.append(File.pathSeparator);
					}
					sb.append("\"");
				}
//				String encoding = (String)project.getSessionProperty(new QualifiedName(null, "encoding"));
//				if (encoding!=null && encoding.trim().length()>0) {
//					sb.append(" -encoding ");
//					sb.append(encoding);
//				}
				
//				IVMInstall jre = JavaRuntime.getDefaultVMInstall(); 
//				File jdkHome = jre.getInstallLocation();
//				String rtpath = jdkHome.getAbsolutePath();
//				rtpath = String.format("%s%s%s%s%s", rtpath, File.separator, "lib", File.separator, "tools.jar");
//				sb.append(rtpath); 
//				System.err.println(sb.toString());
				String commandLine = String.format("-1.5 -warn:-serial %s \"%s\"",
						sb.toString(), resource.getLocation().toString());
				Main.compile(commandLine, out, err);
			}
		});

		return null;
	}



}
